@@ -4,14 +4,23 @@ use JSON qw / from_json /;
 use Module::Load;
 use Moose;
 use Moose::Util qw/ does_role apply_all_roles /;
+use Safe::Isa;
 use Text::Tradition::Collation;
 use Text::Tradition::Error;
 use Text::Tradition::Witness;
-use Text::Tradition::User;
 use TryCatch;
 
 use vars qw( $VERSION );
-$VERSION = "1.0";
+$VERSION = "1.2";
+
+# Enable plugin(s) if available
+eval { with 'Text::Tradition::HasStemma'; };
+# Don't warn normally
+# if( $@ ) {
+# 	warn "Text::Tradition::Analysis not found. Disabling stemma analysis functionality";
+# };
+eval { with 'Text::Tradition::Language'; };
+eval { with 'Text::Tradition::Ownership'; };
 
 has 'collation' => (
     is => 'ro',
@@ -38,12 +47,6 @@ has 'name' => (
     default => 'Tradition',
     );
     
-has 'language' => (
-	is => 'rw',
-	isa => 'Str',
-	predicate => 'has_language',
-	);
-    
 has '_initialized' => (
 	is => 'ro',
 	isa => 'Bool',
@@ -51,32 +54,18 @@ has '_initialized' => (
 	writer => '_init_done',
 	); 
 
-has 'user' => (
-    is => 'rw',
-    isa => 'Text::Tradition::User',
-    required => 0,
-    predicate => 'has_user',
-    clearer => 'clear_user',
-    weak_ref => 1
-    );
-
-has 'public' => (
-    is => 'rw',
-    isa => 'Bool',
-    required => 0,
-    default => sub { 0; },
-    );
-
-# Create the witness before trying to add it
+# Create the witness if necessary before trying to add it
 around 'add_witness' => sub {
     my $orig = shift;
     my $self = shift;
-    # TODO allow add of a Witness object?
-    my %args = @_ == 1 ? %{$_[0]} : @_;
-    $args{'tradition'} = $self;
-    $args{'language'} = $self->language 
-    	if( $self->language && !exists $args{'language'} );
-    my $new_wit = Text::Tradition::Witness->new( %args );
+    my $new_wit;
+    if( @_ == 1 && $_[0]->$_isa( 'Text::Tradition::Witness' ) ) {
+    	$new_wit = shift;
+    } else {
+   		my %args = @_ == 1 ? %{$_[0]} : @_;
+		$args{'tradition'} = $self;
+		$new_wit = Text::Tradition::Witness->new( %args );
+	}
     $self->$orig( $new_wit->sigil => $new_wit );
     return $new_wit;
 };
@@ -100,6 +89,24 @@ around 'witness' => sub {
     return $self->$orig( $arg );
 };
 
+# Cope with witness sigil changes
+sub rename_witness {
+	my( $self, $sig, $newsig ) = @_;
+	my $wit = $self->witness( $sig );
+	$self->throw( "No such witness $sig" ) unless $wit;
+	$self->throw( "Cannot rename witness that has already been collated" )
+		if $wit->is_collated;
+	$wit = $self->del_witness( $sig );
+	try {
+		$wit->_set_sigil( $newsig );
+	} catch ( $e ) {
+		# Don't lose the witness if the rename failed
+		$self->add_witness( $wit );
+		$self->throw( $e );
+	}
+	$self->add_witness( $wit );
+}
+
 =head1 NAME
 
 Text::Tradition - a software model for a set of collated texts
@@ -217,6 +224,7 @@ witness object for the deleted witness.
 
 =begin testing
 
+use TryCatch;
 use_ok( 'Text::Tradition', "can use module" );
 
 my $t = Text::Tradition->new( 'name' => 'empty' );
@@ -242,16 +250,39 @@ if( $wit_a ) {
 is( $s->witness('X'), undef, "There is no witness X" );
 ok( !exists $s->{'witnesses'}->{'X'}, "Witness key X not created" );
 
-my $wit_d = $s->add_witness( 'sigil' => 'D', 'sourcetype' => 'collation' );
+my $wit_d = $s->add_witness( 'sigil' => 'D', 'sourcetype' => 'plaintext',
+	'string' => 'je suis depourvu de foi' );
 is( ref( $wit_d ), 'Text::Tradition::Witness', "new witness created" );
 is( $wit_d->sigil, 'D', "witness has correct sigil" );
 is( scalar $s->witnesses, 4, "object now has four witnesses" );
 
-my $del = $s->del_witness( 'D' );
+try {
+	$s->rename_witness( 'D', 'Invalid Sigil' );
+	ok( 0, "Renamed witness with bad sigil" );
+} catch ( Text::Tradition::Error $e ) {
+	is( $s->witness('D'), $wit_d, "Held onto witness during bad rename" );
+}
+
+try {
+	$s->rename_witness( 'D', 'Q' );
+	ok( 1, "Rename of witness succeeded" );
+	is( $s->witness('Q'), $wit_d, "Witness available under new sigil" );
+	ok( !$s->has_witness('D'), "Witness no longer available under old sigil" );
+} catch ( Text::Tradition::Error $e ) {
+	ok( 0, "Failed to rename witness: " . $e->message );
+}	
+
+my $del = $s->del_witness( 'Q' );
 is( $del, $wit_d, "Deleted correct witness" );
 is( scalar $s->witnesses, 3, "object has three witnesses again" );
 
-# TODO test initialization by witness list when we have it
+try {
+	$s->rename_witness( 'A', 'WitA' );
+	ok( 0, "Successfully renamed an already collated witness" );
+} catch ( Text::Tradition::Error $e ) {
+	is( $e->message, 'Cannot rename witness that has already been collated',
+		"Refused to rename an already-collated witness" );
+}
 
 =end testing
 
@@ -301,6 +332,19 @@ sub BUILD {
     return $self;
 }
 
+=head2 clear_collation
+
+Blow away the existing collation object and mark all witnesses as uncollated.
+Not to be used lightly.
+
+=cut
+
+sub clear_collation {
+	my $self = shift;
+	$self->_save_collation( Text::Tradition::Collation->new( tradition => $self ) );
+	map { $_->is_collated( 0 ) } $self->witnesses;
+}
+
 =head2 add_json_witnesses( $jsonstring, $options )
 
 Adds a set of witnesses from a JSON array specification. This is a wrapper
@@ -320,72 +364,8 @@ sub add_json_witnesses {
 	}
 }
 
-=head1 PLUGIN HOOKS
-
-=head2 enable_stemmata
-
-If the tradition in question does not have the HasStemma role, make it so. Throws
-an error if the role (ergo, if the Analysis package) is not installed.
-
-=cut
-
-sub enable_stemmata {
-	my $self = shift;
-	my $rolename = 'Text::Tradition::HasStemma';
-	return 1 if does_role( $self, $rolename );
-	try {
-		apply_all_roles( $self, $rolename );
-	} catch {
-		throw( "Cannot apply role to enable stemmata; is the Analysis extension installed?" );
-	}
-	return 1;
-}
-
-=head2 enable_morphology
-
-If the tradition in question has readings that do not include the Morphology
-role, apply the role to them. Throws an error if the role (ergo, if the 
-Morphology package) is not installed.
-
-=cut
-
-sub enable_morphology {
-	my $self = shift;
-	my $rolename = 'Text::Tradition::Morphology';
-	try {
-		load( $rolename );
-	} catch {
-		throw( "Cannot apply role to enable morphology; is the extension installed?" );
-	}
-	foreach my $r ( $self->collation->readings ) {
-		apply_all_roles( $r, $rolename )
-			unless does_role( $r, $rolename );
-	}
-	return 1;
-}
-
-=head2 lemmatize
-
-Calls the appropriate lemmatization function for the language of the
-tradition. Implicitly applies the Morphology role where appropriate (and
-throws an error if the package is not installed.)
-
-=cut
-
-# TODO find a better way to hook this
-sub lemmatize {
-	my $self = shift;
-	unless( $self->has_language ) {
-		warn "Please set a language to lemmatize a tradition";
-		return;
-	}
-	$self->enable_morphology;
-	my $mod = "Text::Tradition::Language::" . $self->language;
-	load( $mod );
-	$mod->can( 'lemmatize' )->( $self );
-}
-
 sub throw {
+	my $self = shift;
 	Text::Tradition::Error->throw( 
 		'ident' => 'Tradition error',
 		'message' => $_[0],
