@@ -53,10 +53,6 @@ behavior is to use the first defined text.  If this is not desired,
 use_text should be set to an XPath expression that will select the correct
 text.
 
-=item * language - The name of the applicable L<Text::Tradition::Lang>
-module for language handling. Usually inherited from the language set in
-the L<Text::Tradition> object, and defaults to Default.
-
 =item * identifier - The recognized name of the manuscript, e.g. a library
 identifier. Taken from the msDesc element for a TEI file.
 
@@ -107,10 +103,6 @@ text, if any, of the manuscript.  This should not change after the witness
 has been instantiated, and the path through the collation should always
 match it.
 
-=head2 language
-
-Accessor method to get the witness language.
-
 =head2 identifier
 
 Accessor method for the witness identifier.
@@ -145,6 +137,7 @@ my $ptwit = $trad->add_witness(
 is( ref( $ptwit ), 'Text::Tradition::Witness', 'Created a witness' );
 if( $ptwit ) {
     is( $ptwit->sigil, 'A', "Witness has correct sigil" );
+    $c->make_witness_path( $ptwit );
     is( $c->path_text( $ptwit->sigil ), $str, "Witness has correct text" );
 }
 
@@ -188,6 +181,9 @@ if( $xpwit ) {
 
 =cut
 
+# Enable plugin(s) if available
+eval { with 'Text::Tradition::WitLanguage'; };
+	
 subtype 'SourceType',
 	as 'Str',
 	where { $_ =~ /^(xmldesc|plaintext|json|collation)$/ },
@@ -201,10 +197,10 @@ subtype 'Sigil',
 no Moose::Util::TypeConstraints;
 
 has 'tradition' => (
-	'is' => 'ro',
-	'isa' => 'Text::Tradition',
-	'required' => 1,
-        weak_ref => 1
+	is => 'ro',
+	isa => 'Text::Tradition',
+	required => 1,
+	weak_ref => 1
 	);
 
 # Sigil. Required identifier for a witness, but may be found inside
@@ -216,12 +212,6 @@ has 'sigil' => (
 	writer => '_set_sigil',
 	);
 	
-has 'language' => (
-    is => 'ro',
-    isa => 'Str',
-    default => 'Default',
-    );
-
 # Other identifying information
 has 'identifier' => (
 	is => 'rw',
@@ -291,6 +281,11 @@ has 'layertext' => (
 	predicate => 'has_layertext',
 	);
 	
+has 'is_collated' => (
+	is => 'rw',
+	isa => 'Bool'
+	);
+	
 # Path.	 This is an array of Reading nodes that can be saved during
 # initialization, but should be cleared before saving in a DB.
 has 'path' => (
@@ -328,7 +323,10 @@ sub BUILD {
 		$self->$init_sub();
 		# Remove our XML / source objects; we no longer need them.
 		$self->clear_object if $self->has_object;
-		$self->tradition->collation->make_witness_path( $self );
+		# $self->tradition->collation->make_witness_path( $self );
+	}
+	if( $self->sourcetype eq 'collation' ) {
+		$self->is_collated( 1 );
 	}
 	return $self;
 }
@@ -605,7 +603,7 @@ sub _split_words {
  	my @words;
 	foreach my $w ( @raw_words ) {
 		my $id = $self->sigil . 'r'. $c++;
-		my %opts = ( 'text' => $w, 'id' => $id, 'language' => $self->language );
+		my %opts = ( 'text' => $w, 'id' => $id );
 		my $w_obj = $self->tradition->collation->add_reading( \%opts );
  		# Skip any words that have been canonized out of existence.
 		next if( length( $w_obj->text ) == 0 );
@@ -673,6 +671,9 @@ sub _init_from_json {
 
 sub _init_from_plaintext {
     my( $self ) = @_;
+    unless( $self->has_sigil ) {
+    	throw( "No sigil defined for the plaintext witness" );
+    }
     my $str;
     if( $self->has_file ) {
     	my $ok = open( INPUT, $self->file );
@@ -776,7 +777,7 @@ sub export_as_json {
 		'name' => $self->identifier,
 	};
 	if( $self->is_layered ) {
-		my @lwlist = map { { 't' => $_ || '' } } @{$self->uncorrected};
+		my @lwlist = map { { 't' => $_ || '' } } @{$self->layertext};
 		$obj->{'layertokens'} = \@lwlist;
 	}
 	return $obj;
